/****************************************************************************
**
** Copyright (C) 2003-2006 Tilo Riemer <riemer@crossvc.com>,
**                         Frank Hemer <frank@hemer.org>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** CrossVC is available under two different licenses:
**
** If CrossVC is linked against the GPLed version of Qt 
** CrossVC is released under the terms of GPL also.
**
** If CrossVC is linked against a nonGPLed version of Qt 
** CrossVC is released under the terms of the 
** CrossVC License for non-Unix platforms (CLNU)
**
**
** CrossVC License for non-Unix platforms (CLNU):
**
** Redistribution and use in binary form, without modification, 
** are permitted provided that the following conditions are met:
**
** 1. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 2. It is not permitted to distribute the binary package under a name
**    different than CrossVC.
** 3. The name of the authors may not be used to endorse or promote
**    products derived from this software without specific prior written
**    permission.
** 4. The source code is the creative property of the authors.
**    Extensions and development under the terms of the Gnu Public License
**    are limited to the Unix platform. Any distribution or compilation of 
**    the source code against libraries licensed other than gpl requires 
**    the written permission of the authors.
**
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR 
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
** ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
** WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
**
**
** CrossVC License for Unix platforms:
**
** This program is free software; you can redistribute it and/or modify 
** it under the terms of the GNU General Public License as published by 
** the Free Software Foundation, version 2 of the License.
** This program is distributed in 
** the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
** even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
** PARTICULAR PURPOSE.
**
** See the GNU General Public License version 2 for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
*****************************************************************************/

#include "TextDecoder.h"
#include "TextCodec.h"
#include "globals.h"
#include "LinCVSLog.h"

//----------------------------------------------------------------------------

CTextDecoderPool::CTextDecoderPool(CTextCodecPool *pCodecPool)
   : m_pCodecPool(pCodecPool)
{
   m_decoderDict.setAutoDelete(true);
}

//----------------------------------------------------------------------------

CTextDecoderPool::~CTextDecoderPool()
{
}

//----------------------------------------------------------------------------

void CTextDecoderPool::addDefaultDecoder()
{
   addDecoderForName(TextCodec::DEFAULT_CODEC);
}

//----------------------------------------------------------------------------

void CTextDecoderPool::addDecoderForLocale()
{
   addDecoderForName(nameOfCodecForLocale());
}

//----------------------------------------------------------------------------

void CTextDecoderPool::addDecoderForName(const char* name)
{
   if (!m_decoderDict.find(name)) {//no decoder
      m_pCodecPool->addCodecForName(name);
      m_decoderDict.insert(name, m_pCodecPool->pCodecForName(name)->makeDecoder());
   }
}

//----------------------------------------------------------------------------

QString CTextDecoderPool::nameOfCodecForLocale() const
{
   return m_pCodecPool->nameOfCodecForLocale();
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

CMetaTextDecoder::CMetaTextDecoder(CTextCodecPool *pCodecPool)
{
   m_pDecoderPool = new CTextDecoderPool(pCodecPool);

//TEST
   /*
   m_pDecoderPool->addDefaultDecoder();
   QString def = "Default Codec: ";
   def += TextCodec::DEFAULT_CODEC;
   Debug::g_pLog->log(Debug::LL_INFO, def);

   m_pDecoderPool->addDecoderForLocale();
   QString loc = m_pDecoderPool->nameOfCodecForLocale();

   Debug::g_pLog->log(Debug::LL_INFO, "Codec for locale: " + loc);
   */
}

//----------------------------------------------------------------------------

CMetaTextDecoder::~CMetaTextDecoder()
{
   delete m_pDecoderPool;
}

//----------------------------------------------------------------------------

QString CMetaTextDecoder::decode(const char *pData, const unsigned int size)
{
   return m_pDecoderPool->pDecoderForName(m_generalDecoder)->toUnicode(pData, size);
}

//----------------------------------------------------------------------------

void CMetaTextDecoder::setGeneralDecoder(const char* name)
{
   //wenn die erzeugung irnedeines decoders fehlschlgt, auf 
   //locale oder default zurckfallen... try catch
   if (name == TextCodec::CODEC_FOR_LOCALE) {
      //at least at this point we need try catch because under linux this func fails
      //m_pDecoderPool->addDecoderForLocale();
      //was speichere ich hier? den wirklichen namen des codecs oder die konstante?
      //wenn default codec... kein problem
   } else {
      m_pDecoderPool->addDecoderForName(name);
      m_generalDecoder = name;
   }
}

//----------------------------------------------------------------------------

