/****************************************************************************
**
** Copyright (C) 2004-2006 Frank Hemer <frank@hemer.org>,
**                         Tilo Riemer <riemer@crossvc.com>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** CrossVC is available under two different licenses:
**
** If CrossVC is linked against the GPLed version of Qt 
** CrossVC is released under the terms of GPL also.
**
** If CrossVC is linked against a nonGPLed version of Qt 
** CrossVC is released under the terms of the 
** CrossVC License for non-Unix platforms (CLNU)
**
**
** CrossVC License for non-Unix platforms (CLNU):
**
** Redistribution and use in binary form, without modification, 
** are permitted provided that the following conditions are met:
**
** 1. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 2. It is not permitted to distribute the binary package under a name
**    different than CrossVC.
** 3. The name of the authors may not be used to endorse or promote
**    products derived from this software without specific prior written
**    permission.
** 4. The source code is the creative property of the authors.
**    Extensions and development under the terms of the Gnu Public License
**    are limited to the Unix platform. Any distribution or compilation of 
**    the source code against libraries licensed other than gpl requires 
**    the written permission of the authors.
**
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR 
** IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
** ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
** WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
**
**
** CrossVC License for Unix platforms:
**
** This program is free software; you can redistribute it and/or modify 
** it under the terms of the GNU General Public License as published by 
** the Free Software Foundation, version 2 of the License.
** This program is distributed in 
** the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
** even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
** PARTICULAR PURPOSE.
**
** See the GNU General Public License version 2 for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
*****************************************************************************/

#ifndef UPDATE_H
#define UPDATE_H

#include <qiconset.h>

#include "config/config.h"
#include "CommandInterface.h"
#include "GuardedDir.h"
#include "cvslistview.h"

class Update : public QObject, CommandInterface
{ 
   Q_OBJECT

 signals:
   void deleteObject(QObject *);
   void done(bool);
   void checkTree(DirBase*);
   void checkInProgress(bool);
   void showWarning(const QString&,const QString&);
   void openTreeAndActivate(DirBase *,const QString);

 public:
   static Update * queryUpdateFile(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateFile(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateReplaceFile(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateToDirBranchFile(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateOtherRevisionFile(const QIconSet &, QWidget *, CvsDirListView *, QStringList *);
   static Update * removeLocalOptionsFile(const QIconSet &, QWidget *, CvsDirListView *);

   static Update * queryUpdateDir(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * queryUpdateAllDir(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateDir(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateAllDir(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateReplaceDir(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateReplaceAllDir(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateToDirBranchDir(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * updateOtherRevisionDir(const QIconSet &, QWidget *, CvsDirListView *, QStringList *);
   static Update * updateVirtualDir(const QIconSet &, QWidget *, CvsDirListView *);
   static Update * removeLocalOptionsDir(const QIconSet &, QWidget *, CvsDirListView *);

   static Update * queryUpdateAuto(const QIconSet &, QWidget *, CvsDirListView *, DirBase *, QObject *);
   static Update * queryUpdateAllAuto(const QIconSet &, QWidget *, CvsDirListView *, DirBase *, QObject *);
   static Update * updateAuto(const QIconSet &, QWidget *, CvsDirListView *, DirBase *, QObject *);
   static Update * updateAllAuto(const QIconSet &, QWidget *, CvsDirListView *, DirBase *, QObject *);

   virtual ~Update();

   void afterCall(int,CvsBuffer *,bool);

 protected:
   void cvsCallStarted();
   void cvsCallFinished();

 private slots:
   void acceptCvs();
   void reject(bool);

 private:
   static Update * setup(const QIconSet &, QWidget *, CvsDirListView *, bool, int, QStringList * = NULL);
   static Update * setup(const QIconSet &, QWidget *, CvsDirListView *, DirBase *, bool, int, QStringList * = NULL, QObject * = NULL);

   enum Type { NOOP = 1000, CVS_REVERT_FILES = 1001, CVS_REVERT_DIR = 1002 };
   enum Mode { QUERYFILE, FILE, REPLACEFILE, BRANCHFILE, OTHERFILE, OPTIONSFILE,
	       QUERYDIR, QUERYALLDIR, DIR, ALLDIR, REPLACEDIR, REPLACEALLDIR, BRANCHDIR, OTHERDIR, VIRTUALDIR, OPTIONSDIR };

   Update(const QIconSet &, QWidget *, CvsDirListView *, DirBase *, bool, int, QStringList *);

   void cvsRevertToRevFile(QString = NULL);
   void cvsRevertToRevDir(QString = NULL);
   bool cvsRemoveOptionsOk(QString,bool);

   QIconSet m_whatsThisIconSet;
   QWidget * m_parent;
   CvsDirListView * m_workBench;
   GuardedDir m_dir;

   QWidget * m_dlg;
   bool m_isDir;
   int m_mode;

   QStringList * m_currentTagList;
   QString m_selectedFiles;
   QStringList m_selectedFilesList;
   QStringList m_newFileNameList;
};

#endif

